from cli.allow_commands import AllowCommands
from cli.preprocessor_template import generatedClass
from cli.labelizer_template import generatedClass
from cli.postprocessor_template import generatedClass
from cli.pipeline_template import generatedClass

